function [sys,x0,str,ts] = rst(t,state,input,flag,n,state_0,h)
%RST An M-file S-function for a discrete two-degree-of-freedom
% linear controller 
% 
% R(q)u(k) = T(q)u_c(k) - S(q)y(k)
%
% The polynomials R,S, and T (all of degree n) are inputs and may
% be changed during the simulation.
%
% Inputs: input = [R, S, T, u_c, y]'; controller polynomials R, S,
% and T, setpoint u_c, and process value y
% 
% Outputs: control signal u
%
% States: state = [old_u, old_y, old_u_c, pre_u]'; 
% old control signals, old process values, old setpoints, and
% precomputed control signal
%
% Parameters: n; degree of R,S, and T,
%             state_0; initial state, size(3*n+1)
%             h; sampling interval


switch flag,

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0,
    [sys,x0,str,ts] = mdlInitializeSizes(n,state_0,h);

  %%%%%%%%%%
  % Update %
  %%%%%%%%%%
  case 2,                                                
    sys = mdlUpdate(n,state,input); 

  %%%%%%%%%%
  % Output %
  %%%%%%%%%%
  case 3,                                                
    sys = mdlOutputs(n,state,input);

  %%%%%%%%%%%%%
  % Terminate %
  %%%%%%%%%%%%%
  case {1, 4, 9},                                                
    sys = []; % do nothing

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    error(['unhandled flag = ',num2str(flag)]);
end

%end dsfunc

%
%=======================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=======================================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(n,state_0,h)


sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 3*n+1;
sizes.NumOutputs     = 1;
sizes.NumInputs      = 2+3*(n+1);
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);

x0  = state_0;
str = [];
ts  = [h 0]; 

% end mdlInitializeSizes
%
%=======================================================================
% mdlUpdate
% Handle discrete state updates, sample time hits, and major time step
% requirements.
%=======================================================================
%
function sys = mdlUpdate(n,state,input)

if n == 0 
  sys = [];
else
  
% unpack states and inputs
old_u = state(1:n);
old_y = state(n+1:2*n);
old_u_c = state(2*n+1:3*n);
pre_u = state(3*n+1);

r = input(1:n+1)';
s = input(n+2:2*n+2)';
t = input(2*n+3:3*n+3)';
u_c = input(3*n+4);
y = input(3*n+5);

% update u, y and u_c
u = 1/r(1)*(t(1)*u_c - s(1)*y + pre_u);
u_vec = [u;old_u(1:n-1)];
y_vec = [y;old_y(1:n-1)];
u_c_vec = [u_c;old_u_c(1:n-1)];

% precompute part of u
pre_u = t(2:n+1)*u_c_vec - s(2:n+1)*y_vec - r(2:n+1)*u_vec;

% pack states
state(1:n) = u_vec;
state(n+1:2*n) = y_vec;
state(2*n+1:3*n) = u_c_vec;
state(3*n+1) = pre_u;

sys = state;

end
%end mdlUpdate
%
%=======================================================================
% mdlOutputs
% Return Return the output vector for the S-function
%=======================================================================
%
function sys = mdlOutputs(n,state,input)

if n == 0
  % unpack inputs
  r = input(1);
  s = input(2);
  t = input(3);
  u_c = input(4);
  y = input(5);
  
  % compute u
  u = 1/r*(-s*y + t*u_c);
  
  sys = u;
else
  
  % unpack states and inputs
  pre_u = state(3*n+1);
  r = input(1:n+1)';
  s = input(n+2:2*n+2)';
  t = input(2*n+3:3*n+3)';
  u_c = input(3*n+4);
  y = input(3*n+5);

  % compute u
  u = 1/r(1)*(t(1)*u_c - s(1)*y + pre_u);
  
  sys = u;
end
%end mdlOutputs

