function [sys,x0,str,ts] = rstwind(t,state,input,flag,n,h,A_o,ulim)
%RSTWIND A M-file S-function for a discrete two-degree-of-freedom
% linear controller with anti windup.
%
% A_o(q)v(k)=T(q)u_c(k)-S(q)y(k)-(A_o(q)-R(q))u(t)
% u(k)=f(v(k))
% 
% The polynomials R,S, and T (all of degree n) are inputs and may
% be changed during the simulation.
%
% Inputs: input = [R, S, T, u_c, y]'; controller polynomials R, S,
% and T, setpoint u_c, and process value y
% 
% Outputs: control signal u
%
% States: state = [old_u, old_y, old_u_c, pre_u]'; 
% old control signals, old process values, old setpoints, and
% precomputed control signal
%
% Parameters: n; degree of R,S, and T,
%             h; sampling interval
%             A_o; antiwindup oberserver polynomial
%             ulim; maximum control signal

switch flag,

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0,
    [sys,x0,str,ts] = mdlInitializeSizes(n,h);

  %%%%%%%%%%
  % Update %
  %%%%%%%%%%
  case 2,                                                
    sys = mdlUpdate(n,state,input,A_o,ulim); 

  %%%%%%%%%%
  % Output %
  %%%%%%%%%%
  case 3,                                                
    sys = mdlOutputs(n,state,input,A_o,ulim);

  %%%%%%%%%%%%%
  % Terminate %
  %%%%%%%%%%%%%
  case {1, 4, 9},                                                
    sys = []; % do nothing

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    error(['unhandled flag = ',num2str(flag)]);
end

%end dsfunc

%
%=======================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=======================================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(n,h,A_o)


sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 4*n;
sizes.NumOutputs     = 1;
sizes.NumInputs      = 2+3*(n+1);
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);

x0  = zeros(4*n,1);
str = [];
ts  = [h 0]; 

% end mdlInitializeSizes
%
%=======================================================================
% mdlUpdate
% Handle discrete state updates, sample time hits, and major time step
% requirements.
%=======================================================================
%
function sys = mdlUpdate(n,state,input,A_o,ulim)

if n == 0 
  sys = [];
else
  
% unpack states and inputs
  old_u = state(1:n);
  old_y = state(n+1:2*n);
  old_u_c = state(2*n+1:3*n);
  old_v = state(3*n+1:4*n);

  
  r = input(1:n+1)';
  s = input(n+2:2*n+2)';
  t = input(2*n+3:3*n+3)';
  u_c = input(3*n+4);
  y = input(3*n+5);

  % compute u 
  % A_o(q)v(k)=T(q)u_c(k)-S(q)y(k)-(A_o(q)-R(q))u(t)
  % u(k)=f(v(k))

  v= -A_o(2:n+1)*old_v + t*[u_c ;old_u_c] - s*[y;old_y] + ...
     (A_o(2:n+1)-r(2:n+1))*old_u;

  if v<-ulim
    u = -ulim;
  elseif v<ulim
    u = v;
  else
    u = ulim;
  end;

% pack states
 
old_u = [u;old_u(1:n-1)];
old_y = [y;old_y(1:n-1)];
old_u_c = [u_c;old_u_c(1:n-1)];
old_v = [v;old_v(1:n-1)];

state = [old_u ;old_y ;old_u_c ;old_v];

sys = state;

end
%end mdlUpdate
%
%=======================================================================
% mdlOutputs
% Return Return the output vector for the S-function
%=======================================================================
%
function sys = mdlOutputs(n,state,input,A_o,ulim)

if n == 0
  % unpack inputs
  r = input(1);
  s = input(2);
  t = input(3);
  u_c = input(4);
  y = input(5);
  
  % compute u
  u = 1/r*(-s*y + t*u_c);
  
  sys = u;
else
  
  % unpack states and inputs

  old_u = state(1:n);
  old_y = state(n+1:2*n);
  old_u_c = state(2*n+1:3*n);
  old_v = state(3*n+1:4*n);

  r = input(1:n+1)';
  s = input(n+2:2*n+2)';
  t = input(2*n+3:3*n+3)';
  u_c = input(3*n+4);
  y = input(3*n+5);

  % compute u 
  % A_o(q)v(k)=T(q)u_c(k)-S(q)y(k)-(A_o(q)-R(q))u(t)
  % u(k)=f(v(k))

  v= -A_o(2:n+1)*old_v + t*[u_c ;old_u_c] - s*[y;old_y] + ...
     (A_o(2:n+1)-r(2:n+1))*old_u;

  if v<-ulim
    u = -ulim;
  elseif v<ulim
    u = v;
  else
    u = ulim;
  end;
 
  sys = u;

end
%end mdlOutputs










